#!/bin/bash

# Exit on error
set -e
set -u

# Check args
if [ "$#" -ne 2 ]; then
  echo "Usage: $0 <stage1-participants-json> <stage1-allocations-json>"
  echo "Example: $0 ./stage-participants.json ./stage1-allocations.json"
  exit 1
fi

STAGE1_PARTICIPANTS_FILE="$1"
STAGE1_ALLOCATIONS_FILE="$2"
STAGE1_GENESIS_DIR=stage1-genesis

# Create a temporary target directory
mkdir -p $STAGE1_GENESIS_DIR

# --------

# Copy over the stage1-participants json file for genesis.sh to use
cp $STAGE1_PARTICIPANTS_FILE "$STAGE1_GENESIS_DIR/stage1-participants.json"
echo "Copied over stage1 participants json file to mount dir"

# Copy over the stage1-participants json file for genesis.sh to use
cp $STAGE1_ALLOCATIONS_FILE "$STAGE1_GENESIS_DIR/stage1-allocations.json"
echo "Copied over stage1 allocations json file to mount dir"

# --------

# Run a script with cerc/laconicd:local to generate the genesis file
# with onboarding module state and given allocations
docker run -it \
  -v ./$STAGE1_GENESIS_DIR:/root/.laconicd \
  -v ./scripts:/scripts \
  cerc/laconicd:local bash -c "/scripts/genesis.sh"

# Copy over the genesis file to output folder
OUTPUT_DIR=output
mkdir -p $OUTPUT_DIR
cp ./$STAGE1_GENESIS_DIR/config/genesis.json $OUTPUT_DIR/genesis.json

echo "Genesis file for stage1 written to $OUTPUT_DIR/genesis.json"

# --------

# Clean up
echo "Please remove the temporary data directory: sudo rm -rf $STAGE1_GENESIS_DIR"
